#! /bin/sh
# the next line restarts using wish \
exec /usr/bin/wish "$0" "$@"

# A texture editing utility for POV-Ray, written in Tcl/Tk.
# Code by m2 (c) 2008
# 
# tkxColormap.tcl - Colormap routines for tkxture.tcl
# Version 00.02

# For colormaps, there is a 255-place list.  During creation, the user is allowed to use any
# number of places in this list.  The index seen on screen shows how many are currently being used.

# TBD - Speedup Colormap display.  Faster calculation, update on release of slider.
#
# TBD - Add and delete from any position, not just the end.
# TBD - Colormap levels all require hand-adjustment.  Limit to wall-to-wall behavior by updating neighbors
# if the neighbor's level gets in the way.

# Update the RGB live display.
#
proc updateRGBCanvas { } {
   global nColors varRed varGreen varBlue varColorName

   if { $nColors == 0 } {
      set varRgbHex "D9D9D9"
   } else {
  	   set varRedTemp   [ expr int ( $varRed * 255 ) ]
      set varGreenTemp [ expr int ( $varGreen * 255 ) ]
      set varBlueTemp  [ expr int ( $varBlue * 255 ) ]
      set varRgbHex	[ format %06X [ expr ($varRedTemp << 16) | ($varGreenTemp << 8) | $varBlueTemp ] ]
   }

   set varColorName "0x$varRgbHex"
   set varRgbHex "#$varRgbHex"

	.frameMain.canvasRGB configure -background $varRgbHex
}

# Visual colormap.  Notes for presentation follow.
#
# 1.00
#     Fill with 2nd color.
# 0.nn
#     Interpolate from 1st to 2nd color.
# 0.mm
#     Fill with 1st color.
# 0.00
#
# 0.nn - 0.mm = delta
# 0.60 - 0.20 = 0.40
# The interpolation takes:
# 0.40 of the full height, 
# 0.20 offset of the full height, upward
#
# color1 - color0 = delta
# 0.00 - 0.70 = -0.70
# 
# Color vs current position
# current position / height delta = % of height range. 
# (% of height range * Color1) + (1 - (% of range * Color0))
# 
proc updateColormapCanvas { } {
   global activePattern varRed varGreen varBlue nColors
   global listColormapRed listColormapGreen listColormapBlue 
   global listColormapFilter listColormapTransmit
   global listColormapLevel

#   set canvasWidth [ .frameMain.canvasColormap cget -width ]
#   set canvasHeight [ .frameMain.canvasColormap cget -height ]
#   set canvasWidth   145
#   set canvasHeight  165
#   set canvasWidth [ .frameMain.canvasColormap cget -width ]
#   set canvasHeight [ .frameMain.canvasColormap cget -height ]
   set canvasWidth   150
   set canvasHeight  108
   set canvasBlank #d9d9d9

   set stateMap 0
   set indexColor 0
   # Current position loop, through entire height of colormap display.
   # Use state machine for first, middle, and last colors to speedup.  A switch.
   # First color from zero up to first level (defined by percentage in first level).   
   # 0 First color, meaning from zero up to first transition level.
   # 1 Next colors, interpolate.
   # last color, meaning from last transition level to top.
   #
   # Color vs current position
   # current position / height delta = % of height range. 
   # (% of height range * Color1) + (1 - (% of range * Color0))
   #
   if { $nColors < 2 } { 

      if { $nColors == 0 } {
         set varRgbHex	"#D9D9D9"
      } else {
         set varRedHex   [ expr int ( [ lindex $listColormapRed   $indexColor ] * 255 ) ]
         set varGreenHex [ expr int ( [ lindex $listColormapGreen $indexColor ] * 255 ) ]
         set varBlueHex  [ expr int ( [ lindex $listColormapBlue  $indexColor ] * 255 ) ]
	      set varRgbHex	[ format #%06X [ expr ( $varRedHex << 16 ) | ( $varGreenHex << 8 ) | $varBlueHex ] ]
      }
#      .frameMain.frameColormap.canvasColormap create rectangle 0 0 $canvasWidth $canvasHeight -fill $varRgbHex -outline $varRgbHex
      .frameMain.canvasColormap create rectangle 0 0 $canvasWidth $canvasHeight -fill $varRgbHex -outline $varRgbHex

   } else {
   for { set i 0 } { $i < $canvasHeight } { incr i } {

      set percentHeight [ expr double ( double ( $i ) / double ( $canvasHeight ) ) ]

      switch $stateMap {
         0 {
            if { $percentHeight < [ lindex $listColormapLevel $indexColor ] } {
# v00.01
#               	set varRedHex   [ format 0x%02X [ expr int ( [ lindex $listColormapRed   $indexColor ] * 255 ) ] ]
#	               set varGreenHex [ format 0x%02X [ expr int ( [ lindex $listColormapGreen $indexColor ] * 255 ) ] ]
#	               set varBlueHex  [ format 0x%02X [ expr int ( [ lindex $listColormapBlue  $indexColor ] * 255 ) ] ]

# v00.02 better performance
               	set varRedHex   [ expr int ( [ lindex $listColormapRed   $indexColor ] * 255 ) ]
	               set varGreenHex [ expr int ( [ lindex $listColormapGreen $indexColor ] * 255 ) ]
	               set varBlueHex  [ expr int ( [ lindex $listColormapBlue  $indexColor ] * 255 ) ]

	               set varRgbHex	[ format #%06X [ expr ( $varRedHex << 16 ) | ( $varGreenHex << 8 ) | $varBlueHex ] ]

	               .frameMain.canvasColormap create rectangle 0 $i $canvasWidth [ expr $canvasHeight - $i ] -fill $varRgbHex -outline $varRgbHex
            } else {
               set stateMap 1
            }
         }
         1 {
            if { $percentHeight < [ lindex $listColormapLevel [ expr $indexColor + 1 ] ] } {
               set percentHeightThis [ lindex $listColormapLevel        $indexColor ]
               set percentHeightNext [ lindex $listColormapLevel [ expr ( $indexColor + 1 ) ] ]

               set currentPos [ expr $percentHeight - $percentHeightThis ]
               set percentHeightRange [ expr $currentPos / ( $percentHeightNext - $percentHeightThis ) ]

               set tempRed0   [ lindex $listColormapRed   $indexColor ]
               set tempGreen0 [ lindex $listColormapGreen $indexColor ]
               set tempBlue0  [ lindex $listColormapBlue  $indexColor ]
            
               set tempRed1   [ lindex $listColormapRed   [ expr $indexColor + 1 ] ]
               set tempGreen1 [ lindex $listColormapGreen [ expr $indexColor + 1 ] ]
               set tempBlue1  [ lindex $listColormapBlue  [ expr $indexColor + 1 ] ]

               set tempRed   [ expr ( ( 1.00 - $percentHeightRange ) * $tempRed0   ) + ( $percentHeightRange * $tempRed1   ) ]
               set tempGreen [ expr ( ( 1.00 - $percentHeightRange ) * $tempGreen0 ) + ( $percentHeightRange * $tempGreen1 ) ]
               set tempBlue  [ expr ( ( 1.00 - $percentHeightRange ) * $tempBlue0  ) + ( $percentHeightRange * $tempBlue1  ) ]

# v00.01
#              set varRedHex   [ format 0x%02X [ expr int ( [ expr $tempRed   * 255 ] ) ] ]
#              set varGreenHex [ format 0x%02X [ expr int ( [ expr $tempGreen * 255 ] ) ] ]
#              set varBlueHex  [ format 0x%02X [ expr int ( [ expr $tempBlue  * 255 ] ) ] ]

# v00.02 better performance
             	set varRedHex   [ expr int ( $tempRed   * 255 ) ]
               set varGreenHex [ expr int ( $tempGreen * 255 ) ]
               set varBlueHex  [ expr int ( $tempBlue  * 255 ) ]

               set varRgbHex	[ format #%06X [ expr ( $varRedHex << 16 ) | ( $varGreenHex << 8 ) | $varBlueHex ] ]

               .frameMain.canvasColormap create rectangle 0 [ expr $canvasHeight - $i ] $canvasWidth [ expr $canvasHeight - $i ] -fill $varRgbHex -outline $varRgbHex
            } else { 
               set indexColor [ expr $indexColor + 1 ]
               if { $indexColor == [ expr $nColors - 1 ] } {
                  set stateMap 2
               }
            }
         }
         2 {
# v00.01
#           set varRedHex   [ format 0x%02X [ expr int ( [ lindex $listColormapRed   $indexColor ] * 255 ) ] ]
#           set varGreenHex [ format 0x%02X [ expr int ( [ lindex $listColormapGreen $indexColor ] * 255 ) ] ]
#	         set varBlueHex  [ format 0x%02X [ expr int ( [ lindex $listColormapBlue  $indexColor ] * 255 ) ] ]

# v00.02  better performance
            set varRedHex   [ expr int ( [ lindex $listColormapRed   $indexColor ] * 255 ) ]
            set varGreenHex [ expr int ( [ lindex $listColormapGreen $indexColor ] * 255 ) ]
	         set varBlueHex  [ expr int ( [ lindex $listColormapBlue  $indexColor ] * 255 ) ]

	         set varRgbHex	[ format #%06X [ expr ( $varRedHex << 16 ) | ( $varGreenHex << 8 ) | $varBlueHex ] ]

	         .frameMain.canvasColormap create rectangle 0 [ expr $canvasHeight - $i ] $canvasWidth [ expr $canvasHeight - $i ] -fill $varRgbHex -outline $varRgbHex
         }
         default {
         }
      }
   }
}
}

# end tkxColormap.tcl
