#! /bin/sh
# the next line restarts using wish \
exec /usr/bin/wish "$0" "$@"

# A texture editing utility for POV-Ray, written in Tcl/Tk.
# Code by m2 (c) 2008
# 
# tkxPOVtexture.tcl - buffer editor that makes the POV-Ray texture file.
# Version 00.01

# Main entry.  Buffer the texture include-file script.
proc buffTexture { nameTexture } {
   global bufferTexture activeFile

   set bufferTexture "// $activeFile"
   append bufferTexture "\n#declare  tx$nameTexture = \t\
\ntexture {"
   buffPattern
   buffFinish
   append bufferTexture "\n}"

   buffInterior $nameTexture
   append bufferTexture "\n// end of tkxture.inc"
}

# Buffer the pattern.
proc buffPattern { } {
   global bufferTexture activePattern stateColormap
   global varGradientX varGradientY varGradientZ
   global varTranslateBeforeWarp varTranslateAfterWarp
   global activeImage
   global varImageMapOnce
   global varInterpBilinear varInterpNorm
   global varImageMapPlanar varImageMapSphere varImageMapCylinder varImageMapTorus


   append bufferTexture "\n   pigment {"

   switch $activePattern {

      "color" {
         append bufferTexture "\n      $activePattern" 
         buffPatternColor 
      }
      "crackle" {
         append bufferTexture "\n      $activePattern" 
         buffPatternCrackle
      }
      "gradient" {
         append bufferTexture "\n      $activePattern" 
         if { $varGradientX == 1 } { 
            append bufferTexture " x"
         }
         if { $varGradientY == 1 } { 
            append bufferTexture " y"
         }
         if { $varGradientZ == 1 } { 
            append bufferTexture " z"
         }
      }

      "image map" {
         append bufferTexture "\n      image_map { png \"$activeImage\" "
         if { $varInterpBilinear == 1 } {
            append bufferTexture "\n         interpolate 2"
         }
         if { $varInterpNorm == 1 } {
            append bufferTexture "\n         interpolate 4"
         }
         if { $varImageMapOnce == 1 } {
            append bufferTexture "\n         once"
         }
         if { $varImageMapPlanar } {
            append bufferTexture "\n         map_type 0"
         }
         if { $varImageMapSphere } {
            append bufferTexture "\n         map_type 1"
         }
         if { $varImageMapCylinder } {
            append bufferTexture "\n         map_type 2"
         }
         if { $varImageMapTorus } {
            append bufferTexture "\n         map_type 5"
         }
         append bufferTexture "\n      }" 
      }
      "julia" { 
         buffTextureFractal 
      }
      "mandelbrot" { 
         buffTextureFractal 
      }
      default {
         append bufferTexture "\n      $activePattern" 
      }
   }

   if {  $activePattern != "color" } {
      if { $stateColormap == 0 } {
         buffPatternNonColormap 
      } else { \
         buffPatternColormap 
      }

      if { $varTranslateBeforeWarp == 1 } {
         buffTranslation
      }
      buffTextureShape
      if { $varTranslateAfterWarp == 1 } {
         buffTranslation
      }
   }

   # End build bufferTexture
   append bufferTexture "\n   }"
}

proc buffTranslation {} {
   global bufferTexture varScaleX10 varScaleX varScaleY varScaleZ
   global varRotationX varRotationY varRotationZ

   if { $varScaleX10 == 0 } {
      append bufferTexture "\n      scale < $varScaleX, $varScaleY, $varScaleZ >"
   } else { \
      set tempScaleX [ expr $varScaleX * 10.00 ]
      set tempScaleY [ expr $varScaleY * 10.00 ]
      set tempScaleZ [ expr $varScaleZ * 10.00 ]
      append bufferTexture "\n      scale < $tempScaleX, $tempScaleY, $tempScaleZ >"
   }
   append bufferTexture "\n      rotate < $varRotationX, $varRotationY, $varRotationZ >"
}

# Single color
proc buffPatternColor { } {
   global activePattern bufferTexture
   global varRed varGreen varBlue varFilter varTransmit

   append bufferTexture " rgbft < $varRed, $varGreen, $varBlue, $varFilter, $varTransmit >" 
}

# Non-colormap patterns
# brick, checker, hexagon
proc buffPatternNonColormap { } {
   global bufferTexture
   global activePattern nColors
   global stateNoiseGenerator varNoiseGenerator
   global listColormapRed listColormapGreen listColormapBlue 
   global listColormapFilter listColormapTransmit
   global varBrickX varBrickY varBrickZ varMortar

   if { $stateNoiseGenerator == 1 } {
      append bufferTexture "\n      noise_generator $varNoiseGenerator"
   }

   if {$activePattern != "image map"} {
      for {set i 0} {$i < $nColors} {incr i} {
         set tempR [ lindex $listColormapRed      $i ]
         set tempG [ lindex $listColormapGreen    $i ]
         set tempB [ lindex $listColormapBlue     $i ]
         set tempF [ lindex $listColormapFilter   $i ]
         set tempT [ lindex $listColormapTransmit $i ]

         append bufferTexture "\n      color rgbft < $tempR, $tempG, $tempB, $tempF, $tempT >"
      }
   }
   if { $activePattern == "brick" } {
      append bufferTexture "\n      brick_size < $varBrickX, $varBrickY, $varBrickZ >"
      append bufferTexture "\n      mortar $varMortar"
   }
}

# Colormap patterns
proc buffPatternColormap { } {
   global bufferTexture
   global activePattern nColors
   global stateNoiseGenerator varNoiseGenerator
   global listColormapRed listColormapGreen listColormapBlue 
   global listColormapFilter listColormapTransmit
   global listColormapLevel
   global varFrequency varPhase

   if { $stateNoiseGenerator == 1 } {
      append bufferTexture "\n      noise_generator $varNoiseGenerator"
   }

   append bufferTexture "\n      color_map {"

   for {set i 0} {$i < $nColors} {incr i} {
      set tempL [ lindex $listColormapLevel    $i ]
      set tempR [ lindex $listColormapRed      $i ]
      set tempG [ lindex $listColormapGreen    $i ]
      set tempB [ lindex $listColormapBlue     $i ]
      set tempF [ lindex $listColormapFilter   $i ]
      set tempT [ lindex $listColormapTransmit $i ]

      append bufferTexture "\n         \[ $tempL color rgbft < $tempR, $tempG, $tempB, $tempF, $tempT > ]"
   }
   append bufferTexture    "\n      }"
   append bufferTexture "\n      frequency $varFrequency"
   append bufferTexture "\n      phase $varPhase"
}

# Crackle pattern.
proc buffPatternCrackle { }  {
   global bufferTexture activePattern
   global stateNoiseGenerator varNoiseGenerator
   global varCrackleForm1 varCrackleForm2 varCrackleForm3
   global varCrackleSolid varCrackleMetric varCrackleOffset

   if { $stateNoiseGenerator == 1 } {
      append bufferTexture "\n      noise_generator $varNoiseGenerator"
   }

   append bufferTexture "\n      form < $varCrackleForm1, $varCrackleForm2, $varCrackleForm3 >"
   append bufferTexture "\n      metric $varCrackleMetric"

   if { $varCrackleSolid == 1 } {
      append bufferTexture "\n      solid"
   }
   append bufferTexture "\n      offset $varCrackleOffset"
}

# Fractal texture, julia or mandelbrot 
proc buffTextureFractal { } {
   global bufferTexture activePattern
   global varIterations varFractalExponent
   global varMagnetEnable varMagnetType 
   global varComplexA varComplexB
   global varBumpEnable varBumpSize 
   global varFractalExteriorEnable varExteriorType varExteriorFactor
   global varFractalInteriorEnable varInteriorType varInteriorFactor

   append bufferTexture "\n      "
   if { $varMagnetType != 0 } {
      append bufferTexture "magnet $varMagnetType "
   }
   if { $activePattern == "julia" } {
      append bufferTexture "julia < $varComplexA, $varComplexB > "
   }

   if { $activePattern == "mandelbrot" } {
      append bufferTexture "mandel "
   }

      append bufferTexture "$varIterations"

   if { $varBumpSize != 0 } {
      append bufferTexture ", $varBumpSize"
   }

   if { $varMagnetType == 0 } {
      append bufferTexture "\n      exponent $varFractalExponent"
   }

   if { $varExteriorType != 0 } {
      append bufferTexture "\n      exterior $varExteriorType, $varExteriorFactor"
   }
   if { $varInteriorType != 0 } {
      append bufferTexture "\n      interior $varInteriorType, $varInteriorFactor"
   }
#   buffPatternColormap
#   buffTextureShape
}

# "Shape" of the texture
proc buffTextureShape { } {
   global bufferTexture activePattern activeWarp

   # Black hole
   global varBlackHoleTurbulenceX varBlackHoleTurbulenceY varBlackHoleTurbulenceZ
   global varBlackHoleLocationX varBlackHoleLocationY varBlackHoleLocationZ
   global varBlackHoleRadius varEnableInvertBlackHoleFalloff varBlackHoleFalloff varBlackHoleStrength
   global varBlackHoleRepeatX varBlackHoleRepeatY varBlackHoleRepeatZ

   # Turbulence
   global varAgateTurbulence
   global varWarpEnable
   global varTurbulenceX varTurbulenceY varTurbulenceZ
   global varOctaves varLambda varOmega

   # Translation
   global varScaleX varScaleY varScaleZ varScaleX10
   global varRotationX varRotationY varRotationZ

   # Cylindrical, Spherical, Toroidal
   global varCSTOrientationX varCSTOrientationY varCSTOrientationZ
   global varCSTDistanceExp varToroidalRadius

   # Planar
   global varPlanarNormalX varPlanarNormalY varPlanarNormalZ
   global varPlanarDistance
   
   # Repeat
   global varRepeatDirectionX varRepeatDirectionY varRepeatDirectionZ
   global varRepeatLength varX10BlackHoleRepeat
   global varRepeatFlipX varRepeatFlipY varRepeatFlipZ
   global varRepeatOffsetX varRepeatOffsetY varRepeatOffsetZ
   global varWarpBeforeTurb varWarpAfterTurb

   if { $varWarpEnable == 1 } {
      if { $varWarpAfterTurb == 1 } {
      append bufferTexture "\n      warp {"
      append bufferTexture "\n         turbulence < $varTurbulenceX, $varTurbulenceY, $varTurbulenceZ >"
      append bufferTexture "\n         octaves $varOctaves" 
      append bufferTexture "\n         lambda $varLambda"
      append bufferTexture "\n         omega $varOmega"
      append bufferTexture "\n      }"
      }
      switch $activeWarp {
         "blackhole" {
            append bufferTexture "\n      warp {"
            append bufferTexture "\n         black_hole < $varBlackHoleLocationX, $varBlackHoleLocationY,  $varBlackHoleLocationZ >, $varBlackHoleRadius"
            append bufferTexture "\n         turbulence < $varBlackHoleTurbulenceX, $varBlackHoleTurbulenceY, $varBlackHoleTurbulenceZ >"
            if { $varEnableInvertBlackHoleFalloff == 1 } {
            append bufferTexture "\n         inverse"
            }
            append bufferTexture "\n         falloff $varBlackHoleFalloff"
            append bufferTexture "\n         strength $varBlackHoleStrength"
            if { $varX10BlackHoleRepeat == 1 } {
               set tempBlackHoleRepeatX [ expr $varBlackHoleRepeatX * 10.00 ]
               set tempBlackHoleRepeatY [ expr $varBlackHoleRepeatY * 10.00 ]
               set tempBlackHoleRepeatZ [ expr $varBlackHoleRepeatZ * 10.00 ]
               append bufferTexture "\n         repeat < $tempBlackHoleRepeatX, $tempBlackHoleRepeatY, $tempBlackHoleRepeatZ >"
            } else { \
               append bufferTexture "\n         repeat < $varBlackHoleRepeatX, $varBlackHoleRepeatY, $varBlackHoleRepeatZ >"
            }
            append bufferTexture "\n      }"
         }  
         "cylindrical" {
            append bufferTexture "\n      warp {"
            append bufferTexture "\n         cylindrical orientation " 
            if { $varCSTOrientationX == 1 } { append bufferTexture "< 1, 0, 0 >" } 
            if { $varCSTOrientationY == 1 } { append bufferTexture "< 0, 1, 0 >" } 
            if { $varCSTOrientationZ == 1 } { append bufferTexture "< 0, 0, 1 >" }
            append bufferTexture "\n         dist_exp $varCSTDistanceExp"
            append bufferTexture "\n      }"
         }  
         "planar" {
            append bufferTexture "\n      warp {"
            append bufferTexture "\n         planar < $varPlanarNormalX, $varPlanarNormalY, $varPlanarNormalZ >, $varPlanarDistance"
            append bufferTexture "\n      }"
         }  
         "repeat" {
            append bufferTexture "\n      warp {"
            append bufferTexture "\n         repeat " 
            if { $varRepeatDirectionX == 1 } { append bufferTexture "x " } 
            if { $varRepeatDirectionY == 1 } { append bufferTexture "y " } 
            if { $varRepeatDirectionZ == 1 } { append bufferTexture "z " }
            append bufferTexture "* $varRepeatLength" 
            append bufferTexture "\n         offset < $varRepeatOffsetX, $varRepeatOffsetY, $varRepeatOffsetZ >"
            append bufferTexture "\n         flip < $varRepeatFlipX, $varRepeatFlipY, $varRepeatFlipZ >"
            append bufferTexture "\n      }"
         }  
         "spherical" {
            append bufferTexture "\n      warp {"
            append bufferTexture "\n         spherical orientation "
            if { $varCSTOrientationX == 1 } { append bufferTexture "< 1, 0, 0 >" } 
            if { $varCSTOrientationY == 1 } { append bufferTexture "< 0, 1, 0 >" } 
            if { $varCSTOrientationZ == 1 } { append bufferTexture "< 0, 0, 1 >" } 
            append bufferTexture "\n         dist_exp $varCSTDistanceExp"
            append bufferTexture "\n      }"
         }  
         "toroidal" {
            append bufferTexture "\n      warp {"
            append bufferTexture "\n         toroidal orientation " 
            if { $varCSTOrientationX == 1 } { append bufferTexture "< 1, 0, 0 >" } 
            if { $varCSTOrientationY == 1 } { append bufferTexture "< 0, 1, 0 >" } 
            if { $varCSTOrientationZ == 1 } { append bufferTexture "< 0, 0, 1 >" } 
            append bufferTexture "\n         dist_exp $varCSTDistanceExp"
            append bufferTexture "\n         major_radius $varToroidalRadius"
            append bufferTexture "\n      }"
         }  
         default     {
         }
      }
      if { $varWarpBeforeTurb == 1 } {
      append bufferTexture "\n      warp {"
      append bufferTexture "\n         turbulence < $varTurbulenceX, $varTurbulenceY, $varTurbulenceZ >"
      append bufferTexture "\n         octaves $varOctaves" 
      append bufferTexture "\n         lambda $varLambda"
      append bufferTexture "\n         omega $varOmega"
      append bufferTexture "\n      }"
      }
   } else { \
      if { $activePattern == "agate" } {
         append bufferTexture "\n      agate_turb $varAgateTurbulence"
      } else { \
         append bufferTexture "\n      turbulence $varTurbulenceX"
      }
   }
}

# Finish constraints;
# - Fresnel only works with interior IOR
# 
proc  buffFinish { } {
   global bufferTexture
   global varFinishEnable
   global varAmbientRed varAmbientGreen varAmbientBlue
   global varBrilliance varDiffuse 
   global varConserveEnergy
   global varPhong varPhongSize 
   global varSpecular varRoughness
   global varMetallic
   global varIridescence varIridThickness varIridTurbulence
   global varReflectMin varReflectMax 
   global varFresnelEnable varFalloff varExponent varReflectMetallic

   if { $varFinishEnable == 1 } {
      append bufferTexture "\n   finish {"
      if { $varConserveEnergy == 1 } {
         append bufferTexture "\n      conserve_energy"
      }
      append bufferTexture "\n      ambient rgb < $varAmbientRed, $varAmbientGreen, $varAmbientBlue >"
      append bufferTexture "\n      diffuse $varDiffuse"
      append bufferTexture "\n      brilliance $varBrilliance"
      append bufferTexture "\n      phong $varPhong"
      append bufferTexture "\n      phong_size $varPhongSize"
      append bufferTexture "\n      specular $varSpecular"
      append bufferTexture "\n      roughness $varRoughness"
      append bufferTexture "\n      metallic $varMetallic"
      append bufferTexture "\n      irid { $varIridescence"
      append bufferTexture "\n         thickness $varIridThickness"
      append bufferTexture "\n         turbulence $varIridTurbulence"
      append bufferTexture "\n      }"
      append bufferTexture "\n      reflection {"
      if { $varReflectMin != 0 } {
         append bufferTexture "\n         $varReflectMin, $varReflectMax"
      } else {
         append bufferTexture "\n         $varReflectMax"
      }
      append bufferTexture "\n         fresnel $varFresnelEnable"
      append bufferTexture "\n         falloff $varFalloff"
      if { $varExponent != 0 } {
         append bufferTexture "\n         exponent $varExponent"
      }
      append bufferTexture "\n         metallic $varReflectMetallic"
      append bufferTexture "\n      }"
      append bufferTexture "\n   }"
   } else { \
      append bufferTexture "\n   finish { \t\
\n   }"
   }
}

proc buffInterior { nameTexture } {
   global bufferTexture
   global varInteriorEnable varIor varCaustics
   global varDispersion varDispSamples
   global varFadeDist varFadePower
   global varFadeRed varFadeGreen varFadeBlue

   if { $varInteriorEnable == 1 } {
   
      append bufferTexture "\n \t\
\n#declare in$nameTexture = \t\
\ninterior { \t\
\n   ior $varIor \t\
\n   caustics $varCaustics \t\
\n   dispersion $varDispersion \t\
\n   dispersion_samples $varDispSamples \t\
\n   fade_distance $varFadeDist \t\
\n   fade_power $varFadePower \t\
\n   fade_color < $varFadeRed, $varFadeGreen, $varFadeBlue > \t\
\n}"

   } else { \
      append bufferTexture "\n \t\
\n#declare in$nameTexture = \t\
\ninterior { \t\
\n}"
   }
}


